##############################################
## VfUCFamilySizeRate.r                     ##
##############################################
## Curtis Miller                            ##
## 12/5/14                                  ##
##############################################

## This script calculates the percentage of men and women with numbers of children. It uses
## VfUCGenderGap_RawWageGapData1ACS_2013.csv, so be sure that file is in the working directory.
## It is not a small file, so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Percentage of women with a certain number of children in their household

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages('devtools')
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)

data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS_2013.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

selected.variables = c('STATEFIP', 'SEX', 'PERWT', 'NCHILD', 'AGE')
paste(rep('REPWTP', times = 80), 1:80, sep = '') -> replicate.vars

child.rate.data <- data1[c(selected.variables, replicate.vars)]
child.rate.data <- child.rate.data[ffwhich(child.rate.data, child.rate.data$SEX == 2 &
					   	child.rate.data$AGE >= 18),]

# These vectors will contain the desired rates
child.rate.Utah <- 1:10
child.rate.nat <- 1:10

child.rate.Utah.se <- child.rate.Utah
child.rate.nat.se <- child.rate.nat

calculate.child.rate <- function(x, verbose = FALSE, data, weight = 'PERWT') {
	total <- sum(as.vector(data[[weight]]))
	
	for (i in x) {
		if (verbose == T) {
			cat('Calculating rate for i = ', i, '\n')
		}
		# Select the subset of the data corresponding to 1
		child.data <- data[ffwhich(data, data$NCHILD == i - 1),]
		child.data <- as.vector(child.data[[weight]])
		
		# Calculate the weighted rate
		selected <- sum(child.data)
		
		# Calculate the wage gap and assign to the appropriate location in x
		x[i] <- selected / total
		
	}
	
	return(x)
}

calculate.rate.se <- function(x, estimate, verbose = FALSE, data) {
	sum.square = rep(0, times = length(x))
	
	## The formula for calculating the standard errors is
	## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
	## I calculate first the sum of squares.
	for (i in replicate.vars) {
		if (verbose == T) {
			cat('Calculating replicate for i = ', i, '\n')
		}
		sum.square <- sum.square + (calculate.child.rate(x, data = data, weight = i) - estimate) ^ 2
	}
	
	# Now to calculate the final SE
	x = sqrt(sum.square / 20)
	
	return(x)
}

child.rate.Utah <- calculate.child.rate(child.rate.Utah, verbose = T, data = child.rate.data[ffwhich(child.rate.data, child.rate.data$STATEFIP == 49),])
child.rate.nat <- calculate.child.rate(child.rate.nat, verbose = T, data = child.rate.data)

child.rate.Utah.se <- calculate.rate.se(child.rate.Utah.se, verbose = T, data = child.rate.data[ffwhich(child.rate.data, child.rate.data$STATEFIP == 49),], estimate = child.rate.Utah)
child.rate.nat.se <- calculate.rate.se(child.rate.nat.se, verbose = T, data = child.rate.data, estimate = child.rate.nat)

child.rate.Utah.mean <- sum(child.rate.Utah * 0:9)
child.rate.nat.mean <- sum(child.rate.nat * 0:9)

child.rate.data.frame <- data.frame(Rate = c(child.rate.Utah, child.rate.nat) * 100,
				    Children = rep(0:9, times = 2),
				    Level = rep(c('Utah', 'National'), each = length(child.rate.Utah)),
				    SE = c(child.rate.Utah.se, child.rate.nat.se) * 100)

write.csv(child.rate.data.frame, file = 'VfUCChildRate.csv')

# Create the plot
ggplot(child.rate.data.frame) +
	geom_ribbon(aes(x = Children, ymin = Rate - SE, ymax = Rate + SE, fill = Level), alpha = 0.3) +
	geom_line(aes(x = Children, y = Rate, color = Level), size = 1.1) +
	geom_vline(xintercept = c(child.rate.Utah.mean, child.rate.nat.mean), color = c('#00BFC4', '#F8766D')) +
	ggtitle("Rate of Number of Children Living \n With Women 18 and Older") +
	ylab('Rate (as %)') +
	scale_x_continuous(breaks = c(0:9, round(c(child.rate.Utah.mean, child.rate.nat.mean), digits = 2))) +
	theme_bw() +
	theme(plot.title = element_text(size = rel(2)),
	      axis.text.x = element_text(angle = 35, hjust = 1, vjust = 1))

# Save plots in working directory
ggsave('VfUCChildRateWomen.png', width = 5.5, height = 4, units = 'in', scale = 1.5)
ggsave('VfUCChildRateWomen.wmf', width = 5.5, height = 4, units = 'in', scale = 1.5)

# Let's create a plot only for mothers, including the mean number of children of mothers

child.rate.Utah.mothers <- child.rate.Utah[2:10] / sum(child.rate.Utah[2:10])
child.rate.nat.mothers <- child.rate.nat[2:10] / sum(child.rate.nat[2:10])

child.Utah.mothers.mean <- sum(child.rate.Utah.mothers * 1:9)
child.nat.mothers.mean <- sum(child.rate.nat.mothers * 1:9)

child.rate.mothers.data.frame <- data.frame(Rate = c(child.rate.Utah.mothers, child.rate.nat.mothers) * 100,
				    Children = rep(1:9, times = 2),
				    Level = rep(c('Utah', 'National'), each = length(child.rate.Utah.mothers)))

write.csv(child.rate.mothers.data.frame, file = 'VfUCChildRateMothers.csv')

# Create the plot
ggplot(child.rate.mothers.data.frame) +
	geom_bar(aes(x = Children, y = Rate, fill = Level), stat = 'identity', position = position_dodge()) +
#	geom_vline(xintercept = c(child.Utah.mothers.mean, child.nat.mothers.mean), color = c('#00BFC4', '#F8766D')) +
	ggtitle("Frequency of Number of Children \n Living With Mothers 18 and Older, 2013") +
	ylab('Rate (as %)') +
	scale_x_continuous(breaks = 1:9) +
	theme_bw() +
	theme(plot.title = element_text(size = rel(2)),
	      axis.text.x = element_text(angle = 35, hjust = 1, vjust = 1))

# Save plots in working directory
ggsave('VfUCChildRateMothers.png', width = 5.5, height = 4, units = 'in', scale = 1.5)
ggsave('VfUCChildRateMothers.wmf', width = 5.5, height = 4, units = 'in', scale = 1.5)
