##############################################
## VfUCGenderGapAge.r                       ##
##############################################
## Curtis Miller                            ##
## 11/3/14                                  ##
##############################################

## This script calculates the labor participation rate for women in Utah in 2014. It uses
## VfUCGenderGap_RawWageGapData1ACS_2013.csv, so be sure that file is in the working directory.
## It is not a small file, so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Gender gap by age

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages(devtools)
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)
#library(beepr)
library(matrixStats)	# This contains a function for calculating weighted medians, weigthedMedian()

# Load a new CSV file that contains the replicate weights variables
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS_2013.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

# Create a vector containing the names of the 80 REPWTPXX vars
paste(rep('REPWTP', times = 80), 1:80, sep = '') -> replicate.vars
selected.variables <- c('STATEFIP', 'SEX', 'EMPSTAT', 'AGE', 'UHRSWORK', 'WKSWORK2', 'PERWT', 'INCEARN')
age.gap.data <- data1[c(selected.variables, replicate.vars)]

# Select full-time year-round workers who are employed, older than 16, who's sex is no NIU (this is the population I will analyze)
age.gap.data <- age.gap.data[ffwhich(age.gap.data, age.gap.data$SEX != 9 &
                              age.gap.data$EMPSTAT == 1 &
                              age.gap.data$AGE >= 16 &
                              age.gap.data$UHRSWORK >= 35 &
                              age.gap.data$WKSWORK2 == 6),]

# Clean up age.gap.data so unneeded variables are removed
selected.variables <- c('STATEFIP', 'SEX', 'AGE', 'PERWT', 'INCEARN')
age.gap.data <- age.gap.data[c(selected.variables, replicate.vars)]

# This variable will contain the age earnings gap; first, it contains the left end point of the class
age.gap.Utah <- seq(from = 15, to = 60, by = 5)
age.gap.nat <- age.gap.Utah

age.gap.Utah.se <- age.gap.Utah
age.gap.nat.se <- age.gap.nat

## calculate.age.gap calculates the earnings gap of women and men (based on weighted median earnings
## in each group), taking in a vector containing the left end point of the classes (which are five
## years wide), an ff data object (data), the name of the variable to represent the weight (weight),
## and whether to print progress (verbose). x should be a sequence containing the left endpoints
## (usually starting at 15). data could be restricted to be just Utah, or national. The resulting
## ratios are returned by the function.

calculate.age.gap <- function(x, data, weight = 'PERWT', verbose = FALSE) {
  age.data <- as.data.frame(data[ffwhich(data, data$SEX == 1),])
  men.med.wage <- weightedMedian(age.data$INCEARN, w = age.data[[weight]])
  for (i in x) {
    # Make verbose
    if (verbose == T) {
      cat('Calculating medians for i = ', i, '\n')
    }
    
    # Select the subset of the data corresponding to the selected age; all individiauls with age between i and i + 5
    age.data <- as.data.frame(data[ffwhich(data, data$AGE >= i & data$AGE < i + 5), c('INCEARN', 'SEX', weight)])

    # Calculate the weighted median wage for women and men
    women.med.wage <- weightedMedian(age.data$INCEARN[which(age.data$SEX == 2)], w = age.data[which(age.data$SEX == 2), weight])
    
    # Calculate the wage gap and assign to the appropriate location in x
    x[which(x == i)] <- women.med.wage / men.med.wage
  }
  rm(age.data)
  
  return(x)
}

## This function takes a row from, say, race.wage.gap.lat.se and a
## subset of race.gap.data.2013. It calculates standard errors using the REPWPTXX
## variables and assigns them to the passed row of the table. x is a vector,
## verbose a logical value, and data an ff object. estimate is a vector that
## contains the estimated value for which a standard error will be calculated.

calculate.gap.se <- function(x, estimate, verbose = FALSE, data) {
  sum.square = rep(0, times = length(x))
  
  ## The formula for calculating the standard errors is
  ## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
  ## I calculate first the sum of squares.
  for (i in replicate.vars) {
    if (verbose == T) {
      cat('Calculating replicate for i = ', i, '\n')
    }
    sum.square <- sum.square + (calculate.age.gap(x, data = data, weight = i) - estimate) ^ 2
  }
  
  # Now to calculate the final SE
  x = sqrt(sum.square / 20)
  
  return(x)
}

# Calculate the earnings gap and standard errors
age.gap.Utah <- calculate.age.gap(age.gap.Utah, data = age.gap.data[ffwhich(age.gap.data, age.gap.data$STATEFIP == 49),], verbose = T)
age.gap.nat <- calculate.age.gap(age.gap.nat, data = age.gap.data, verbose = T)

age.gap.Utah.se <- calculate.gap.se(age.gap.Utah.se, data = age.gap.data[ffwhich(age.gap.data, age.gap.data$STATEFIP == 49),], verbose = T, estimate = age.gap.Utah)
age.gap.nat.se <- calculate.gap.se(age.gap.nat.se, data = age.gap.data, verbose = T, estimate = age.gap.nat)

# Create a dataframe containing the data, and save it to a csv file
age.gap.data.frame <- data.frame(Gap = c(age.gap.Utah, age.gap.nat) * 100,
                                 SE = c(age.gap.Utah.se, age.gap.nat.se) * 100,
                                 Age = rep(c(16, seq(from = 20, to = 60, by = 5)), times = 2),
                                 Level = c(rep('Utah', times = length(age.gap.Utah)), rep('National', times = length(age.gap.nat))))

write.csv(age.gap.data.frame, file = 'VfUCGenderGapAge2.csv')

# Create the plot
ggplot(age.gap.data.frame, aes(x = Age, y = Gap, fill = Level)) +
  geom_bar(size = 1.1, stat = 'identity', position = position_dodge()) +
  geom_errorbar(aes(ymin = Gap - SE, ymax = Gap + SE), position=position_dodge(3.5), width = 2) +
  ggtitle("Women's Share of All Men's Earnings in 2013 \n Among Full-Time Year-Round Workers, By Age") +
  ylab('Share (as %)') +
  scale_x_continuous(breaks = c(16, seq(from = 20, to = 60, by = 5)), labels = c('16-19', '20-24', '25-29', '30-34', '35-39', '40-44', '45-49', '50-54', '55-59', '60-64')) +
  theme_bw() +
  theme(plot.title = element_text(size = rel(2)),
        axis.text.x = element_text(angle = 35, hjust = 1, vjust = 1))

# Save plots in working directory
ggsave('VfUCGenderGapAge2.png', width = 5.5, height = 4, units = 'in', scale = 1.5)
ggsave('VfUCGenderGapAge2.wmf', width = 5.5, height = 4, units = 'in', scale = 1.5)
