##############################################
## VfUCGenderGapEducDetailed.r              ##
##############################################
## Curtis Miller                            ##
## 11/3/14                                  ##
##############################################

## This script calculates the labor participation rate for women in Utah in 2014. It uses
## VfUCGenderGap_RawWageGapData1ACS_2013.csv, so be sure that file is in the working directory.
## It is not a small file, so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Gender gap by years of education

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages(devtools)
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)
#library(beepr)
library(matrixStats)	# This contains a function for calculating weighted medians, weigthedMedian()

# Load a new CSV file that contains the replicate weights variables
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS_2013.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

# Create a vector containing the names of the 80 REPWTPXX vars
paste(rep('REPWTP', times = 80), 1:80, sep = '') -> replicate.vars
selected.variables <- c('STATEFIP', 'SEX', 'EMPSTAT', 'AGE', 'UHRSWORK', 'WKSWORK2', 'PERWT', 'INCEARN', 'EDUCD')

educ.gap.data <- data1[c(selected.variables, replicate.vars)]

# Select full-time year-round workers who are employed, older than 16, who's sex is no NIU (this is the population I will analyze)
educ.gap.data <- educ.gap.data[ffwhich(educ.gap.data, educ.gap.data$SEX != 9 &
				     	educ.gap.data$EMPSTAT == 1 &
				     	educ.gap.data$AGE >= 16 &
				     	educ.gap.data$UHRSWORK >= 35 &
				     	educ.gap.data$WKSWORK2 == 6 &
				     	educ.gap.data$EDUCD != 1),]

# Clean up educ.gap.data so unneeded variables are removed
selected.variables <- c('STATEFIP', 'SEX', 'PERWT', 'INCEARN', 'EDUCD')
educ.gap.data <- educ.gap.data[c(selected.variables, replicate.vars)]

# This variable will contain the education earnings gap; first, it contains the left end point of the class
educ.gap.Utah <- c(061, 063, 064, 065, 071, 081, 101, 114, 115)
educ.gap.nat <- educ.gap.Utah

educ.gap.Utah.se <- educ.gap.Utah
educ.gap.nat.se <- educ.gap.nat

## calculate.educ.gap calculates the earnings gap of women and men (based on weighted median earnings
## in each group), taking in a vector containing the left end point of the classes (which are five
## years wide), an ff data object (data), the name of the variable to represent the weight (weight),
## and whether to print progress (verbose). x should be a sequence containing the left endpoints
## (usually starting at 15). data could be restricted to be just Utah, or national. The resulting
## ratios are returned by the function.

calculate.educ.gap <- function(x, data, weight = 'PERWT', verbose = FALSE) {
	for (i in x) {
		# Make verbose
		if (verbose == T) {
			cat('Calculating medians for i = ', i, '\n')
		}
		
		# Select the subset of the data corresponding to the selected education
    if (i == 61) {
		  educ.data <- as.data.frame(data[ffwhich(data, data$EDUCD <= i), c('INCEARN', 'SEX', weight)])
    }
    else {
      educ.data <- as.data.frame(data[ffwhich(data, data$EDUCD == i), c('INCEARN', 'SEX', weight)])
    }
    
		# Calculate the weighted median earnings for women and men
		women.med.wage <- weightedMedian(educ.data$INCEARN[which(educ.data$SEX == 2)], w = educ.data[which(educ.data$SEX == 2), weight])
		men.med.wage <- weightedMedian(educ.data$INCEARN[which(educ.data$SEX == 1)], w = educ.data[which(educ.data$SEX == 1), weight])
		
		# Calculate the wage gap and assign to the appropriate location in x
		x[which(x == i)] <- women.med.wage / men.med.wage
	}
	rm(educ.data)
	
	return(x)
}

## This function takes a row from, say, race.wage.gap.lat.se and a
## subset of race.gap.data.2013. It calculates standard errors using the REPWPTXX
## variables and assigns them to the passed row of the table. x is a vector,
## verbose a logical value, and data an ff object. estimate is a vector that
## contains the estimated value for which a standard error will be calculated.

calculate.gap.se <- function(x, estimate, verbose = FALSE, data) {
	sum.square = rep(0, times = length(x))
	
	## The formula for calculating the standard errors is
	## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
	## I calculate first the sum of squares.
	for (i in replicate.vars) {
		if (verbose == T) {
			cat('Calculating replicate for i = ', i, '\n')
		}
		sum.square <- sum.square + (calculate.educ.gap(x, data = data, weight = i) - estimate) ^ 2
	}
	
	# Now to calculate the final SE
	x = sqrt(sum.square / 20)
	
	return(x)
}

# Calculate the earnings gap and standard errors
educ.gap.Utah <- calculate.educ.gap(educ.gap.Utah, data = educ.gap.data[ffwhich(educ.gap.data, educ.gap.data$STATEFIP == 49),], verbose = T)
educ.gap.nat <- calculate.educ.gap(educ.gap.nat, data = educ.gap.data, verbose = T)

educ.gap.Utah.se <- calculate.gap.se(educ.gap.Utah.se, data = educ.gap.data[ffwhich(educ.gap.data, educ.gap.data$STATEFIP == 49),], verbose = T, estimate = educ.gap.Utah)
educ.gap.nat.se <- calculate.gap.se(educ.gap.nat.se, data = educ.gap.data, verbose = T, estimate = educ.gap.nat)

educd.codes <- read.csv('EducCodesDetailed.csv')
educd.labels <- as.character(educd.codes$Educational.attainment[which(educd.codes$EDUCD %in% c(061, 063, 064, 065, 071, 081, 101, 114, 115))])
educd.labels[1] = 'Less than high school diploma'

# Create a dataframe containing the data, and save it to a csv file
educ.gap.data.frame <- data.frame(Gap = c(educ.gap.Utah, educ.gap.nat) * 100,
				 SE = c(educ.gap.Utah.se, educ.gap.nat.se) * 100,
				 Educ = rep(educd.labels, times = 2),
				 Level = c(rep('Utah', times = length(educ.gap.Utah)), rep('National', times = length(educ.gap.nat))))

write.csv(educ.gap.data.frame, file = 'VfUCGenderGapEducDetailed.csv')

# Create the plot
ggplot(educ.gap.data.frame) +
  geom_point(aes(x = Gap, y = Educ, color = Level), size = 4) +
  geom_errorbarh(aes(x = Gap, y = Educ, xmin = Gap - SE, xmax = Gap + SE, color = Level), linetype = 'twodash') +
  scale_y_discrete(limits = rev(as.character(educd.labels))) +
  ggtitle("Women's Share of Men's Earnings in 2013 Among \n Full-Time Year-Round Workers, By Level of Education") +
  xlab('Share (as %)') +
  ylab('Level of Education') +
  theme_bw() +
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        panel.grid.major.y = element_line(color = 'grey60', linetype = 'dotted'),
        axis.text.x = element_text(angle = 35, hjust = 1, vjust = 1),
        plot.title = element_text(size = rel(2)))

# Save plots in working directory
ggsave('VfUCGenderGapEducDetailed.png', width = 9, height = 3.5, units = 'in', scale = 1.5)
ggsave('VfUCGenderGapEducDetailed.wmf', width = 9, height = 3.5, units = 'in', scale = 1.5)
