##############################################
## VfUCGenderGapIndustry.r                ##
##############################################
## Curtis Miller                            ##
## 11/4/14                                  ##
##############################################

## This script calculates the earnings gap within industries in Utah. It uses
## VfUCGenderGap_RawWageGapData1ACS_2013.csv, so be sure that file is in the working directory.
## It is not a small file, so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Gender gap by industry

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages('devtools')
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)
#library(beepr)
library(matrixStats)  # This contains a function for calculating weighted medians, weigthedMedian()

# Load a new CSV file that contains the replicate weights variables
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS_2013.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

# Create a vector containing the names of the 80 REPWTPXX vars
paste(rep('REPWTP', times = 80), 1:80, sep = '') -> replicate.vars
selected.variables <- c('STATEFIP', 'SEX', 'IND', 'PERWT', 'INCEARN', 'AGE', 'UHRSWORK', 'WKSWORK2')
ind.gap.data <- data1[c(selected.variables, replicate.vars)]

# Select workers who are employed, older than 16, who's sex is no NIU (this is the population I will analyze)
ind.gap.data <- ind.gap.data[ffwhich(ind.gap.data, ind.gap.data$SEX != 9 &
				     	ind.gap.data$AGE >= 16 &
				     	ind.gap.data$IND > 0 &
				     	ind.gap.data$IND < 9920 &
				     	ind.gap.data$UHRSWORK >= 35 &
				     	ind.gap.data$WKSWORK2 == 6),]

# Clean up ind.gap.data so unneeded variables are removed
selected.variables <- c('STATEFIP', 'SEX', 'PERWT', 'INCEARN', 'IND')
ind.gap.data <- ind.gap.data[c(selected.variables, replicate.vars)]

# Load a file containing IND codes and the equivalent INDBROAD equivalent
ind.broad.cross <- read.csv('INDCodes.csv')
indbroad.codebk <- read.csv('INDBROADCodebook.csv')

# Create a column containing broad industry codes
ind.gap.data$INDBROAD = as.ff(rep(0, times = length(ind.gap.data$IND)))
crosswalkINDBROAD <- function(i) {
	return(ind.broad.cross$INDBROAD[ind.broad.cross$IND.CODE == i])
}
ind.gap.data$INDBROAD <- ffvecapply(sapply(ind.gap.data$IND[i1:i2], crosswalkINDBROAD), X = ind.gap.data$IND, RETURN = T, VERBOSE = T, BATCHSIZE = 50000)

# Vectors that will contain the desired gaps
ind.gap.Utah <- 1:16
ind.gap.nat <- 1:16

ind.gap.Utah.se <- 1:16
ind.gap.nat.se <- 1:16

## calculate.age.gap calculates the earnings gap of women and men (based on weighted median earnings
## in each group), taking in a vector containing the INDBROAD codes an ff data object (data), the
## name of the variable to represent the weight (weight), and whether to print progress (verbose). x
## should be a sequence containing the codes of the desired industries to calculate. data could be
## restricted to be just Utah, or national. The resulting ratios are returned by the function.

calculate.ind.gap <- function(x, data, weight = 'PERWT', verbose = FALSE) {
	for (i in x) {
		# Make verbose
		if (verbose == T) {
			cat('Calculating medians for i = ', i, '\n')
		}
		
		# Select the subset of the data corresponding to the selected industry group
		ind.data <- as.data.frame(data[ffwhich(data, data$INDBROAD == i), c('INCEARN', 'SEX', weight)])
		
		# Calculate the weighted median wage for women and men
		women.med.wage <- weightedMedian(ind.data$INCEARN[which(ind.data$SEX == 2)], w = ind.data[which(ind.data$SEX == 2), weight])
		men.med.wage <- weightedMedian(ind.data$INCEARN[which(ind.data$SEX == 1)], w = ind.data[which(ind.data$SEX == 1), weight])
		
		# Calculate the wage gap and assign to the appropriate location in x
		x[which(x == i)] <- women.med.wage / men.med.wage
	}
	rm(ind.data)
	
	return(x)
}

## This function takes a row from, say, race.wage.gap.lat.se and a
## subset of race.gap.data.2013. It calculates standard errors using the REPWPTXX
## variables and assigns them to the passed row of the table. x is a vector,
## verbose a logical value, and data an ff object. estimate is a vector that
## contains the estimated value for which a standard error will be calculated.

calculate.gap.se <- function(x, estimate, verbose = FALSE, data) {
	sum.square = rep(0, times = length(x))
	
	## The formula for calculating the standard errors is
	## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
	## I calculate first the sum of squares.
	for (i in replicate.vars) {
		if (verbose == T) {
			cat('Calculating replicate for i = ', i, '\n')
		}
		sum.square <- sum.square + (calculate.ind.gap(x, data = data, weight = i) - estimate) ^ 2
	}
	
	# Now to calculate the final SE
	x = sqrt(sum.square / 20)
	
	return(x)
}

ind.gap.Utah <- calculate.ind.gap(ind.gap.Utah, data = ind.gap.data[ffwhich(ind.gap.data, ind.gap.data$STATEFIP == 49),], verbose = T)
ind.gap.nat <- calculate.ind.gap(ind.gap.nat, data = ind.gap.data, verbose = T)

ind.gap.Utah.se <- calculate.gap.se(ind.gap.Utah.se, data = ind.gap.data[ffwhich(ind.gap.data, ind.gap.data$STATEFIP == 49),], verbose = T, estimate = ind.gap.Utah)
ind.gap.nat.se <- calculate.gap.se(ind.gap.nat.se, data = ind.gap.data, verbose = T, estimate = ind.gap.nat)

# Create a dataframe containing the data, and save it to a csv file
ind.gap.data.frame <- data.frame(Gap = c(ind.gap.Utah, ind.gap.nat) * 100,
				 SE = c(ind.gap.Utah.se, ind.gap.nat.se) * 100,
				 Ind = rep(indbroad.codebk$BROAD.INDUSTRY.NAME[2:17], times = 2),
				 Level = c(rep('Utah', times = length(ind.gap.Utah)), rep('National', times = length(ind.gap.nat))))

write.csv(ind.gap.data.frame, file = 'VfUCGenderGapInd.csv')

ggplot(ind.gap.data.frame) +
	geom_point(aes(x = Gap, y = reorder(Ind, Gap), color = Level), size = 4) +
	geom_errorbarh(aes(x = Gap, y = reorder(Ind, Gap), xmin = Gap - SE, xmax = Gap + SE, color = Level), linetype = 'twodash') +
	ggtitle("Women's Share of Men's Earnings in 2013 Among \n Full-Time Year-Round Workers, By Industry") +
	xlab('Share (as %)') +
  xlim(0, 125) +
	ylab('Industry') +
	theme_bw() +
	theme(panel.grid.major.x = element_blank(),
	      panel.grid.minor.x = element_blank(),
	      panel.grid.major.y = element_line(color = 'grey60', linetype = 'dotted'),
	      axis.text.x = element_text(angle = 35, hjust = 1, vjust = 1),
	      plot.title = element_text(size = rel(2)))

# Save plots in working directory
ggsave('GenderGapIndustry.png', width = 11, height = 5.5, units = 'in', scale = 1.5)
ggsave('GenderGapIndustry.wmf', width = 11, height = 5.5, units = 'in', scale = 1.5)
