##############################################
## VfUCGenderGapMaritalMom.r                ##
##############################################
## Curtis Miller                            ##
## 11/10/14                                 ##
##############################################

## This script calculates the earnings gap within parental/marital status. It uses
## VfUCGenderGap_RawWageGapData1ACS_2013.csv, so be sure that file is in the working directory.
## It is not a small file, so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Gender gap by marital and parental status

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages('devtools')
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)
#library(beepr)
library(matrixStats)  # This contains a function for calculating weighted medians, weigthedMedian()

# Load a new CSV file that contains the replicate weights variables
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS_2013.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

# Create a vector containing the names of the 80 REPWTPXX vars
paste(rep('REPWTP', times = 80), 1:80, sep = '') -> replicate.vars
selected.variables <- c('STATEFIP', 'SEX', 'PERWT', 'INCEARN', 'AGE', 'EMPSTAT', 'MARST', 'NCHILD', 'UHRSWORK', 'WKSWORK2')
mom.gap.data <- data1[c(selected.variables, replicate.vars)]

# Select workers who are employed, older than 16, who's sex is no NIU (this is the population I will analyze), and no NAs in MARST and NCHILD
mom.gap.data <- mom.gap.data[ffwhich(mom.gap.data, mom.gap.data$SEX != 9 &
				     	mom.gap.data$AGE >= 16 &
				     	mom.gap.data$EMPSTAT == 1 &
				     	mom.gap.data$UHRSWORK >= 35 &
				     	mom.gap.data$WKSWORK2== 6),]

# Clean up mom.gap.data so unneeded variables are removed
selected.variables <- c('STATEFIP', 'SEX', 'PERWT', 'INCEARN', 'MARST', 'NCHILD')
mom.gap.data <- mom.gap.data[c(selected.variables, replicate.vars)]

# Vectors that will contain the desired gaps; each entry corresponds to: 1, Married (MARST 1, 2, and 3); 2, Not Married (MARST 4 and 5); 3, Never Married (MARST 6); 4, mother (NCHILD > 0); 5, not a mother (NCHILD == 0); and 6, all women
mom.gap.Utah <- 1:6
mom.gap.nat <- 1:6

mom.gap.Utah.se <- 1:6
mom.gap.nat.se <- 1:6

## calculate.mom.gap calculates the earnings gap of women and men (based on weighted median earnings
## in each group), taking in a vector containing the six groups, an ff data object (data), the
## name of the variable to represent the weight (weight), and whether to print progress (verbose). x
## should be a sequence containing the codes of the desired occupations to calculate. data could be
## restricted to be just Utah, or national. The resulting ratios are returned by the function.

calculate.mom.gap <- function(x, data, weight = 'PERWT', verbose = FALSE) {
	mom.data <- as.data.frame(data[, c('INCEARN', 'SEX', weight)])
	men.med.wage <- weightedMedian(mom.data$INCEARN[which(mom.data$SEX == 1)], w = mom.data[which(mom.data$SEX == 1), weight])
	
	i = 1
	if (verbose == T) {
		cat('Calculating medians for i = ', i, '\n')
	}
	# Select the subset of the data corresponding to 1
	mom.data <- as.data.frame(data[ffwhich(data, data$MARST >= 1 & data$MARST <= 3), c('INCEARN', 'SEX', weight)])
		
	# Calculate the weighted median wage for women and men
	women.med.wage <- weightedMedian(mom.data$INCEARN[which(mom.data$SEX == 2)], w = mom.data[which(mom.data$SEX == 2), weight])
		
	# Calculate the wage gap and assign to the appropriate location in x
	x[i] <- women.med.wage / men.med.wage
	i <- i + 1
	
	
	# Select the subset of the data corresponding to 2
	if (verbose == T) {
		cat('Calculating medians for i = ', i, '\n')
	}
	mom.data <- as.data.frame(data[ffwhich(data, data$MARST >= 4 & data$MARST <= 5), c('INCEARN', 'SEX', weight)])
	
	# Calculate the weighted median wage for women and men
	women.med.wage <- weightedMedian(mom.data$INCEARN[which(mom.data$SEX == 2)], w = mom.data[which(mom.data$SEX == 2), weight])
	
	# Calculate the wage gap and assign to the appropriate location in x
	x[i] <- women.med.wage / men.med.wage
	i <- i + 1
	
	
	# Select the subset of the data corresponding to 3
	if (verbose == T) {
		cat('Calculating medians for i = ', i, '\n')
	}
	mom.data <- as.data.frame(data[ffwhich(data, data$MARST == 6), c('INCEARN', 'SEX', weight)])
	
	# Calculate the weighted median wage for women and men
	women.med.wage <- weightedMedian(mom.data$INCEARN[which(mom.data$SEX == 2)], w = mom.data[which(mom.data$SEX == 2), weight])
	
	# Calculate the wage gap and assign to the appropriate location in x
	x[i] <- women.med.wage / men.med.wage
	i <- i + 1
	
	
	# Select the subset of the data corresponding to 4
	if (verbose == T) {
		cat('Calculating medians for i = ', i, '\n')
	}
	mom.data <- as.data.frame(data[ffwhich(data, data$NCHILD > 0), c('INCEARN', 'SEX', weight)])
	
	# Calculate the weighted median wage for women and men
	women.med.wage <- weightedMedian(mom.data$INCEARN[which(mom.data$SEX == 2)], w = mom.data[which(mom.data$SEX == 2), weight])
	
	# Calculate the wage gap and assign to the appropriate location in x
	x[i] <- women.med.wage / men.med.wage
	i <- i + 1
	
	
	# Select the subset of the data corresponding to 5
	if (verbose == T) {
		cat('Calculating medians for i = ', i, '\n')
	}
	mom.data <- as.data.frame(data[ffwhich(data, data$NCHILD == 0), c('INCEARN', 'SEX', weight)])
	
	# Calculate the weighted median wage for women and men
	women.med.wage <- weightedMedian(mom.data$INCEARN[which(mom.data$SEX == 2)], w = mom.data[which(mom.data$SEX == 2), weight])
	
	# Calculate the wage gap and assign to the appropriate location in x
	x[i] <- women.med.wage / men.med.wage
	i <- i + 1
	
	
	# Select the subset of the data corresponding to 6
	if (verbose == T) {
		cat('Calculating medians for i = ', i, '\n')
	}
	mom.data <- as.data.frame(data[, c('INCEARN', 'SEX', weight)])
	
	# Calculate the weighted median wage for women and men
	women.med.wage <- weightedMedian(mom.data$INCEARN[which(mom.data$SEX == 2)], w = mom.data[which(mom.data$SEX == 2), weight])
	
	# Calculate the wage gap and assign to the appropriate location in x
	x[i] <- women.med.wage / men.med.wage
	
	rm(mom.data)
	
	return(x)
}

## This function takes a row from, say, race.wage.gap.lat.se and a
## subset of race.gap.data.2013. It calculates standard errors using the REPWPTXX
## variables and assigns them to the passed row of the table. x is a vector,
## verbose a logical value, and data an ff object. estimate is a vector that
## contains the estimated value for which a standard error will be calculated.

calculate.gap.se <- function(x, estimate, verbose = FALSE, data) {
	sum.square = rep(0, times = length(x))
	
	## The formula for calculating the standard errors is
	## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
	## I calculate first the sum of squares.
	for (i in replicate.vars) {
		if (verbose == T) {
			cat('Calculating replicate for i = ', i, '\n')
		}
		sum.square <- sum.square + (calculate.mom.gap(x, data = data, weight = i) - estimate) ^ 2
	}
	
	# Now to calculate the final SE
	x = sqrt(sum.square / 20)
	
	return(x)
}

mom.gap.Utah <- calculate.mom.gap(mom.gap.Utah, data = mom.gap.data[ffwhich(mom.gap.data, mom.gap.data$STATEFIP == 49),], verbose = T)
mom.gap.nat <- calculate.mom.gap(mom.gap.nat, data = mom.gap.data, verbose = T)

mom.gap.Utah.se <- calculate.gap.se(mom.gap.Utah.se, data = mom.gap.data[ffwhich(mom.gap.data, mom.gap.data$STATEFIP == 49),], verbose = T, estimate = mom.gap.Utah)
mom.gap.nat.se <- calculate.gap.se(mom.gap.nat.se, data = mom.gap.data, verbose = T, estimate = mom.gap.nat)

# Create a dataframe containing the data, and save it to a csv file
mom.gap.data.frame <- data.frame(Gap = c(mom.gap.Utah, mom.gap.nat) * 100,
				 SE = c(mom.gap.Utah.se, mom.gap.nat.se) * 100,
				 Group = rep(c('Married', 'Unmarried', 'Never Married', 'Children Present', 'Children Not Present', 'All'), times = 2),
				 Level = c(rep('Utah', times = length(mom.gap.Utah)), rep('National', times = length(mom.gap.nat))))

write.csv(mom.gap.data.frame, file = 'VfUCGenderGapMom2.csv')

ggplot(mom.gap.data.frame) +
	geom_point(aes(x = Gap, y = reorder(Group, Gap), color = Level), size = 4) +
	geom_errorbarh(aes(x = Gap, y = reorder(Group, Gap), xmin = Gap - SE, xmax = Gap + SE, color = Level), linetype = 'twodash') +
	ggtitle("Women's Share of Men's Earnings in 2013 Among Full-Time \n Year-Round Workers, By Marital/Parental Status") +
	xlab('Share (as %)') +
	ylab('Marital/Parental Group') +
	theme_bw() +
	theme(panel.grid.major.x = element_blank(),
	      panel.grid.minor.x = element_blank(),
	      panel.grid.major.y = element_line(color = 'grey60', linetype = 'dotted'),
	      axis.text.x = element_text(angle = 35, hjust = 1, vjust = 1),
	      plot.title = element_text(size = rel(2)))

# Save plots in working directory
ggsave('GenderGapMaritalParental2.png', width = 7.5, height = 3, units = 'in', scale = 1.5)
ggsave('GenderGapMaritalParental2.wmf', width = 7.5, height = 3, units = 'in', scale = 1.5)
