##############################################
## VfUCGenderGapOccupation.r                ##
##############################################
## Curtis Miller                            ##
## 11/4/14                                  ##
##############################################

## This script calculates the earnings gap within occupations in Utah. It uses
## VfUCGenderGap_RawWageGapData1ACS_2013.csv, so be sure that file is in the working directory.
## It is not a small file, so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Gender gap by occupation

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages('devtools')
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)
#library(beepr)
library(matrixStats)  # This contains a function for calculating weighted medians, weigthedMedian()

# Load a new CSV file that contains the replicate weights variables
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS_2013.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

# Create a vector containing the names of the 80 REPWTPXX vars
paste(rep('REPWTP', times = 80), 1:80, sep = '') -> replicate.vars
selected.variables <- c('STATEFIP', 'SEX', 'OCC', 'PERWT', 'INCEARN', 'AGE', 'UHRSWORK', 'WKSWORK2')
occ.gap.data <- data1[c(selected.variables, replicate.vars)]

# Select workers who are employed, older than 16, who's sex is no NIU (this is the population I will analyze)
occ.gap.data <- occ.gap.data[ffwhich(occ.gap.data, occ.gap.data$SEX != 9 &
                                       occ.gap.data$AGE >= 16 &
                                       occ.gap.data$OCC > 0 &
                                       occ.gap.data$OCC < 9920 &
				       occ.gap.data$UHRSWORK >= 35 &
				       occ.gap.data$WKSWORK2 == 6),]

# Clean up occ.gap.data so unneeded variables are removed
selected.variables <- c('STATEFIP', 'SEX', 'PERWT', 'INCEARN', 'OCC')
occ.gap.data <- occ.gap.data[c(selected.variables, replicate.vars)]

# Load a file containing OCC codes and the equivalent OCCBROAD equivalent
occ.broad.cross <- read.csv('OCCCodes.csv')
occbroad.codebk <- read.csv('OCCBROADCodebook.csv')

# Create a column containing broad occupation codes
occ.gap.data$OCCBROAD = as.ff(rep(0, times = length(occ.gap.data$OCC)))
crosswalkOCCBROAD <- function(i) {
	return(occ.broad.cross$OCCBROAD[occ.broad.cross$X2010.2012.ACS.OCC.code == i])
}
occ.gap.data$OCCBROAD <- ffvecapply(sapply(occ.gap.data$OCC[i1:i2], crosswalkOCCBROAD), X = occ.gap.data$OCC, RETURN = T, VERBOSE = T, BATCHSIZE = 50000)

# Vectors that will contain the desired gaps
occ.gap.Utah <- 1:23
occ.gap.nat <- 1:23

occ.gap.Utah.se <- 1:23
occ.gap.nat.se <- 1:23

## calculate.age.gap calculates the earnings gap of women and men (based on weighted median earnings
## in each group), taking in a vector containing the OCCBROAD codes an ff data object (data), the
## name of the variable to represent the weight (weight), and whether to print progress (verbose). x
## should be a sequence containing the codes of the desired occupations to calculate. data could be
## restricted to be just Utah, or national. The resulting ratios are returned by the function.

calculate.occ.gap <- function(x, data, weight = 'PERWT', verbose = FALSE) {
	for (i in x) {
		# Make verbose
		if (verbose == T) {
			cat('Calculating medians for i = ', i, '\n')
		}
		
		# Select the subset of the data corresponding to the selected occupation group
		occ.data <- as.data.frame(data[ffwhich(data, data$OCCBROAD == i), c('INCEARN', 'SEX', weight)])
		
		# Calculate the weighted median wage for women and men
		women.med.wage <- weightedMedian(occ.data$INCEARN[which(occ.data$SEX == 2)], w = occ.data[which(occ.data$SEX == 2), weight])
		men.med.wage <- weightedMedian(occ.data$INCEARN[which(occ.data$SEX == 1)], w = occ.data[which(occ.data$SEX == 1), weight])
		
		# Calculate the wage gap and assign to the appropriate location in x
		x[which(x == i)] <- women.med.wage / men.med.wage
	}
	rm(occ.data)
	
	return(x)
}

## This function takes a row from, say, race.wage.gap.lat.se and a
## subset of race.gap.data.2013. It calculates standard errors using the REPWPTXX
## variables and assigns them to the passed row of the table. x is a vector,
## verbose a logical value, and data an ff object. estimate is a vector that
## contains the estimated value for which a standard error will be calculated.

calculate.gap.se <- function(x, estimate, verbose = FALSE, data) {
	sum.square = rep(0, times = length(x))
	
	## The formula for calculating the standard errors is
	## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
	## I calculate first the sum of squares.
	for (i in replicate.vars) {
		if (verbose == T) {
			cat('Calculating replicate for i = ', i, '\n')
		}
		sum.square <- sum.square + (calculate.occ.gap(x, data = data, weight = i) - estimate) ^ 2
	}
	
	# Now to calculate the final SE
	x = sqrt(sum.square / 20)
	
	return(x)
}

occ.gap.Utah <- calculate.occ.gap(occ.gap.Utah, data = occ.gap.data[ffwhich(occ.gap.data, occ.gap.data$STATEFIP == 49),], verbose = T)
occ.gap.nat <- calculate.occ.gap(occ.gap.nat, data = occ.gap.data, verbose = T)

occ.gap.Utah.se <- calculate.gap.se(occ.gap.Utah.se, data = occ.gap.data[ffwhich(occ.gap.data, occ.gap.data$STATEFIP == 49),], verbose = T, estimate = occ.gap.Utah)
occ.gap.nat.se <- calculate.gap.se(occ.gap.nat.se, data = occ.gap.data, verbose = T, estimate = occ.gap.nat)

# Create a dataframe containing the data, and save it to a csv file
occ.gap.data.frame <- data.frame(Gap = c(occ.gap.Utah, occ.gap.nat) * 100,
				 SE = c(occ.gap.Utah.se, occ.gap.nat.se) * 100,
				 Occ = rep(occbroad.codebk$BROAD.OCCUPATION.NAME[2:24], times = 2),
				 Level = c(rep('Utah', times = length(occ.gap.Utah)), rep('National', times = length(occ.gap.nat))))

write.csv(occ.gap.data.frame, file = 'VfUCGenderGapOcc.csv')

ggplot(occ.gap.data.frame) +
	geom_point(aes(x = Gap, y = reorder(Occ, Gap), color = Level), size = 4) +
	geom_errorbarh(aes(x = Gap, y = reorder(Occ, Gap), xmin = Gap - SE, xmax = Gap + SE, color = Level), linetype = 'twodash') +
	ggtitle("Women's Share of Men's Earnings in 2013 Among \n Full-Time Year-Round Workers, By Occupation") +
	xlab('Share (as %)') +
	ylab('Occupation') +
	theme_bw() +
	theme(panel.grid.major.x = element_blank(),
	      panel.grid.minor.x = element_blank(),
	      panel.grid.major.y = element_line(color = 'grey60', linetype = 'dotted'),
	      axis.text.x = element_text(angle = 35, hjust = 1, vjust = 1),
	      plot.title = element_text(size = rel(2)))

# Save plots in working directory
ggsave('GenderGapOccupation.png', width = 11, height = 5.5, units = 'in', scale = 1.5)
ggsave('GenderGapOccupation.wmf', width = 11, height = 5.5, units = 'in', scale = 1.5)
