##################################################
## VfUCRawGenderGapPartTimeEarningsRaw.r	##
##################################################
## Curtis Miller        	                ##
## 10/16/14                               	##
##################################################

## This script calculates Utah's gender gap (both over time and compared to the rest of the
## nation). It uses VfUCGenderGap_RawWageGapData1ACS.csv, so be sure that file is in the
## working directory, and anticipate to spend several minutes waiting for data to load.
## It is not a small file, so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Raw gender gap in earnings across time for part time workers
##    * Raw gender gap in earnings across states for part time workers
##    * (Others coming soon)

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages(devtools)
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)
#library(beepr)
library(matrixStats)	# This contains a function for calculating weighted medians, weigthedMedian()

# Read in the data from both files for manipulation
data1 <- read.csv.ffdf(file = 'VUCGenderGapDataCPSPartTime.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)
paste(rep('REPWTP', times = 160), 1:160, sep = '') -> replicate.vars

raw.gap.data.2014 <- data1
# First, load a CSV file containing state FIP codes
state.fip.code <- read.csv('StateFipCodes.csv')
# Only states are wanted, not regions or unidentified; this selects the desired subset (including DC)
state.fip.code <- state.fip.code[1:51,]

raw.wage.gap.lat = 1:51
raw.wage.gap.lat.se <- raw.wage.gap.lat

raw.gap.data.2014 <- as.data.frame(raw.gap.data.2014[ffwhich(raw.gap.data.2014, raw.gap.data.2014$SEX != 9 &
						       	raw.gap.data.2014$EARNWEEK > 0 &
							raw.gap.data.2014$EARNWEEK < 9999.99 &
						       	raw.gap.data.2014$AGE >= 16 &
							raw.gap.data.2014$UHRSWORK < 35),])

# For each state, calculate the wage gap and assign to a vector
calculate.raw.gap.2014 <- function(x, verbose = FALSE, weight = 'EARNWT') {
	for (i in 1:51) {
		# Make verbose
		if (verbose == T) {
			cat('Calculating medians for i = ', i, '\n')
		}
		
		# Calculate the weighted median wage for women and men
		women.med.wage <- weightedMedian(raw.gap.data.2014$EARNWEEK[which(raw.gap.data.2014$SEX == 2 & raw.gap.data.2014$STATEFIP == state.fip.code$STATEFIP[i])], w = raw.gap.data.2014[which(raw.gap.data.2014$SEX == 2 & raw.gap.data.2014$STATEFIP == state.fip.code$STATEFIP[i]), weight])
		men.med.wage <- weightedMedian(raw.gap.data.2014$EARNWEEK[which(raw.gap.data.2014$SEX == 1 & raw.gap.data.2014$STATEFIP == state.fip.code$STATEFIP[i])], w = raw.gap.data.2014[which(raw.gap.data.2014$SEX == 1 & raw.gap.data.2014$STATEFIP == state.fip.code$STATEFIP[i]), weight])
	
		# Calculate the wage gap and assign to the appropriate location in raw.wage.gap.lat
		x[which(x == i)] <- women.med.wage / men.med.wage
	}
	
	return(x)
}

calculate.gap.se <- function(x, estimate, verbose = FALSE) {
	sum.square = rep(0, times = length(x))
	
	## The formula for calculating the standard errors is
	## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
	## I calculate first the sum of squares.
	for (i in replicate.vars) {
		if (verbose == T) {
			cat('Calculating replicate for i = ', i, '\n')
		}
		sum.square <- sum.square + (calculate.raw.gap.2014(x, weight = i) - estimate) ^ 2
	}
	
	# Now to calculate the final SE
	x = sqrt(sum.square / 40)
	
	return(x)
}
#beep()

raw.wage.gap.lat <- calculate.raw.gap.2014(raw.wage.gap.lat, verbose = T)
raw.wage.gap.lat.se <- calculate.gap.se(raw.wage.gap.lat.se, estimate = raw.wage.gap.lat, verbose = T)

raw.wage.gap.nat <- weightedMedian(raw.gap.data.2014$EARNWEEK[which(raw.gap.data.2014$SEX == 2)], w = raw.gap.data.2014$EARNWT[which(raw.gap.data.2014$SEX == 2)]) / weightedMedian(raw.gap.data.2014$EARNWEEK[which(raw.gap.data.2014$SEX == 1)], w = raw.gap.data.2014$EARNWT[which(raw.gap.data.2014$SEX == 1)])

rm(raw.gap.data.2014)

# Create a data frame containing the data that can be used by ggplot2
raw.wage.gap.lat.data <- data.frame(Gap = raw.wage.gap.lat * 100, SE = raw.wage.gap.lat.se * 100, State = as.character(state.fip.code$State[1:51]))
raw.wage.gap.lat.data$hl <- 'no'
raw.wage.gap.lat.data$hl[raw.wage.gap.lat.data$State == 'Utah'] <- 'yes'

write.csv(raw.wage.gap.lat.data, 'WageGapPartTimeWagesLat.csv')

ggplot(raw.wage.gap.lat.data, aes(y = Gap, x = reorder(State, Gap), fill = hl)) +
	geom_bar(stat = 'identity') +
	geom_crossbar(aes(x = State, y = Gap, ymin = Gap - SE, ymax = Gap + SE), color = NA, fill = 'pink', alpha = .5) +
	scale_fill_manual(values = c('grey20', '#00BFC4'), guide = F) +
  	geom_hline(yintercept = raw.wage.gap.nat * 100, color = '#F8766D', linetype = 'dashed', size = 1) +
	xlab('State') +
	ylab('Share (as %)') +
	geom_text(aes(label = round(Gap, digits = 1)), vjust = 1.5, size = 2.5, color = 'white') +
	geom_text(label = 'National estimate', x = 3, y = round(raw.wage.gap.nat * 100), vjust = -0.5, size = 4) +
	scale_y_continuous(breaks = c(seq(0, 100, by = 25), round(raw.wage.gap.nat * 100, digits = 1))) +
	theme_bw() +
	ggtitle("Women's Share of Men's Weekly Earnings \n Among Part-Time Workers From 2010 to 2014") +
	theme(plot.title = element_text(size = rel(2)),
	      axis.text.x = element_text(angle = 45, hjust = 1, vjust = 1))

# Save plots in working directory
ggsave('WageGapPartTimeWagesLat.png', width = 9, height = 6, units = 'in', scale = 1.5)
ggsave('WageGapPartTimeWagesLat.wmf', width = 9, height = 6, units = 'in', scale = 1.5)
