######################################
## VfUCGenderGapParticipationRate.r ##
######################################
## Curtis Miller                    ##
## 10/8/14                          ##
######################################

## This script calculates the labor participation rate for women in Utah in 2014. It uses
## VfUCGenderGap_RawWageGapData1.csv, so be sure that file is in the working directory. It
## is not a small file, so the packages ff and ffbase are used to extract data.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Participation rate across time
##    * Participation rate across states
##    * (Others coming soon)

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')

library(ff)
library(ffbase)
library(ggplot2)

# Read in the data from both files for manipulation
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1CPS.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)
data2 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData2CPS.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)
selected.variables = c('YEAR', 'STATEFIP', 'SEX', 'EMPSTAT')
# Combine the data to be analyzed that is present in both files into one object
partic.data <- ffdfappend(data1[selected.variables], data2[selected.variables])
#rm(data); gc()
# women.partic.rate.long will contain the participation rate of women in the labor force across years; same with men.partic.rate.long
women.partic.rate.long = 1962:2014
men.partic.rate.long = 1962:2014

for (i in 1962:2014) {
  year.data <- partic.data[ffwhich(partic.data, partic.data$YEAR == i),]
  year.data <- as.data.frame(year.data); gc()
  # Restrict to the subset of Utah data and individuals that are not NIU; for the years 1968 to 1976, Utah is not coded separately, so regional proxies are used.
  if (i <= 1972 & i >= 1968) {
  	state.code = 79
  } else if (i <= 1976 & i >= 1973) {
  	state.code = 89
  } else state.code = 49
  
  year.data <- year.data[which((year.data$STATEFIP == state.code) & (year.data$SEX != 9) & (year.data$EMPSTAT != 00)),]
  
  women.labor <- length(which(year.data$SEX == 2 & year.data$EMPSTAT %in% c(1,10,12,20,21,22)))
  women.total <- length(which(year.data$SEX == 2))
  men.labor <- length(which(year.data$SEX == 1 & year.data$EMPSTAT %in% c(1,10,12,20,21,22)))
  men.total <- length(which(year.data$SEX == 1))
  
  # Store participation rates in women.partic.rate.long and men.partic.rate.long at appropriate location
  women.partic.rate.long[which(women.partic.rate.long == i)] <- women.labor / women.total
  men.partic.rate.long[which(men.partic.rate.long == i)] <- men.labor / men.total
}

## I want to make a line graph for the participation rates, so we will create a data frame with year and the sexes participation rate,
## then plot using ggplot2.

partic.rate.long.data <- data.frame(Rate = c(women.partic.rate.long, men.partic.rate.long), Year = c(1962:2014, 1962:2014), Sex = c(rep(c('Female'), times = length(women.partic.rate.long)), rep(c('Male'), times = length(women.partic.rate.long))))

# I want to be able to highlight recessions: the following data frame contains the beginning and end dates of recessions, along with parameters for the min and max coordinates of the box. The fractions represent month number minus one (so Jan is 0 and Dec is 11).
recessions.draw.rect <- data.frame(begin = c(1969+11/12, 1973+10/12, 1980, 1981+6/12, 1990+6/12, 2001+2/12, 2007+11/12), end = c(1970+11/12, 1975+2/12, 1980+6/12, 1982+10/12, 1991+2/12, 2001+10/12, 2009+5/12), ymin = -Inf, ymax = Inf)
# I also want a rectangle to show the years when a regional proxy is being used
proxy.rect <- data.frame(xmin = 1968, xmax = 1976, ymin = -Inf, ymax = Inf)
ggplot(partic.rate.long.data, aes(x = Year, y = Rate, colour = Sex)) +
	geom_line(size = 1.1) +
	geom_rect(data = recessions.draw.rect, aes(xmin = begin, xmax = end, ymin = ymin, ymax = ymax), color = 'grey10', linetype = 0, alpha = 0.5, inherit.aes = F) +
	geom_rect(data = proxy.rect, aes(xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax), linetype = 0, alpha = 0.2, inherit.aes = F) +
	ggtitle('Participation Rate in the Labor Force') +
	theme_bw() +
	theme(plot.title = element_text(size = rel(2)))

## Let's now do a latitudinal examination of participation rate in the labor force. Restrict the sample to the 2014 data,
## then calculate the sexes' participation rate across states.

# First, load a CSV file containing state FIP codes
state.fip.code <- read.csv('StateFipCodes.csv')
# Only states are wanted, not regions or unidentified; this selects the desired subset (including DC)
state.fip.code <- state.fip.code[1:51,]

women.partic.rate.lat = 1:51
men.partic.rate.lat = 1:51

partic.data.2014 <- as.data.frame(partic.data[ffwhich(partic.data, partic.data$YEAR == 2014 & (partic.data$SEX != 9) & (partic.data$EMPSTAT != 00)),])

# For each state, calculate each sex's participation rate and assign to a vector
for (i in 1:51) {
	women.labor <- length(which(partic.data.2014$STATEFIP == state.fip.code$STATEFIP[i] & partic.data.2014$SEX == 2 & partic.data.2014$EMPSTAT %in% c(1,10,12,20,21,22)))
	women.total <- length(which(partic.data.2014$STATEFIP == state.fip.code$STATEFIP[i] & partic.data.2014$SEX == 2))
	men.labor <- length(which(partic.data.2014$STATEFIP == state.fip.code$STATEFIP[i] & partic.data.2014$SEX == 1 & partic.data.2014$EMPSTAT %in% c(1,10,12,20,21,22)))
	men.total <- length(which(partic.data.2014$STATEFIP == state.fip.code$STATEFIP[i] & partic.data.2014$SEX == 1))
	
	# Store participation rates in women.partic.rate.long and men.partic.rate.long at appropriate location
	women.partic.rate.lat[i] <- women.labor / women.total
	men.partic.rate.lat[i] <- men.labor / men.total
}

# Create a data frame containing the data that can be used by ggplot2
partic.rate.lat.data <- data.frame(Rate = c(women.partic.rate.lat, men.partic.rate.lat), State = c(as.character(state.fip.code$State[1:51]), as.character(state.fip.code$State[1:51])), Sex = c(rep(c('Female'), times = length(women.partic.rate.lat)), rep(c('Male'), times = length(women.partic.rate.lat))))
ggplot(partic.rate.lat.data, aes(x = Rate, y = reorder(State, Rate), color = Sex)) +
	geom_point(size = 4) +
	theme_bw() +
	ggtitle('Participation Rate in the Labor Force') +
	ylab('State') +
	theme(panel.grid.major.x = element_blank(),
	      panel.grid.minor.x = element_blank(),
	      panel.grid.major.y = element_line(color = 'grey60', linetype = 'dotted'),
	      plot.title = element_text(size = rel(2)))
