##########################################
## VfUCRawGenderGapRaw.r		##
##########################################
## Curtis Miller        	        ##
## 10/16/14                             ##
##########################################

## This script calculates Utah's gender gap (both over time and compared to the rest of the
## nation). It uses VfUCGenderGap_RawWageGapData1ACS.csv, so be sure that file is in the
## working directory, and anticipate to spend several minutes waiting for data to load.
## It is not a small file, so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Raw gender gap across time
##    * Raw gender gap across states
##    * (Others coming soon)

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages(devtools)
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)
#library(beepr)
library(matrixStats)	# This contains a function for calculating weighted medians, weigthedMedian()

# Read in the data from both files for manipulation
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

selected.variables = c('YEAR', 'STATEFIP', 'SEX', 'EMPSTAT', 'AGE', 'PERWT', 'INCEARN')
raw.gap.data <- data1[selected.variables]
# raw.wage.gap.nat.long will contain the raw wage gap over selected census years; raw.wage.gap.Utah.long is the same for Utah
#census.years <- c(1850, 1860, 1870, 1880, 1900, 1910, 1920, 1930, 1940, 1950, 1960, 1970, 1980, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012)
census.years <- c(2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012)
raw.wage.gap.Utah.long <- census.years
raw.wage.gap.nat.long <- census.years

for (i in census.years) {
	# Make verbose
	cat('Calculating medians for i = ', i, '\n')
	# Select the subset of the data corresponding to the selected year, sex that isn't NIU, employed, and older than 16; see codebook for codes
	year.data <- raw.gap.data[ffwhich(raw.gap.data, raw.gap.data$YEAR == i &
																			raw.gap.data$SEX != 9 &
																			raw.gap.data$EMPSTAT == 1 &
																			raw.gap.data$AGE >= 16),]
	year.data <- as.data.frame(year.data); gc()
	
	# Calculate the weighted median wage for women and men
	women.med.wage <- weightedMedian(year.data$INCEARN[which(year.data$SEX == 2)], w = year.data$PERWT[which(year.data$SEX == 2)])
	men.med.wage <- weightedMedian(year.data$INCEARN[which(year.data$SEX == 1)], w = year.data$PERWT[which(year.data$SEX == 1)])
	
	# Calculate the wage gap and assign to the appropriate location in raw.wage.gap.nat.long
	raw.wage.gap.nat.long[which(raw.wage.gap.nat.long == i)] <- women.med.wage / men.med.wage
	
	# Repeat for Utah
	year.data <- year.data[which(year.data$STATEFIP == 49),]
	
	women.med.wage <- weightedMedian(year.data$INCEARN[which(year.data$SEX == 2)], w = year.data$PERWT[which(year.data$SEX == 2)])
	men.med.wage <- weightedMedian(year.data$INCEARN[which(year.data$SEX == 1)], w = year.data$PERWT[which(year.data$SEX == 1)])
	
	raw.wage.gap.Utah.long[which(raw.wage.gap.Utah.long == i)] <- women.med.wage / men.med.wage
}
#beep()
rm(year.data)

## I want to make a line graph for the participation rates, so we will create a data frame with year and the sexes participation rate,
## then plot using ggplot2.

raw.wage.gap.long.data <- data.frame(Gap = c(raw.wage.gap.nat.long * 100, raw.wage.gap.Utah.long * 100), Year = c(census.years, census.years), Level = c(rep('National', times = length(raw.wage.gap.nat.long)), rep('Utah', times = length(raw.wage.gap.Utah.long))))

# I want to be able to highlight recessions: the following data frame contains the beginning and end dates of recessions, along with parameters for the min and max coordinates of the box. The fractions represent month number minus one (so Jan is 0 and Dec is 11).
#recessions.draw.rect <- data.frame(begin = c(1969+11/12, 1973+10/12, 1980, 1981+6/12, 1990+6/12, 2001+2/12, 2007+11/12), end = c(1970+11/12, 1975+2/12, 1980+6/12, 1982+10/12, 1991+2/12, 2001+10/12, 2009+5/12), ymin = -Inf, ymax = Inf)
ggplot(raw.wage.gap.long.data, aes(x = Year, y = Gap, color = Level)) +
	geom_line(size = 1.1) +
	ylim(50, 100) +
	ggtitle("Utah Women's Share of Men's Earnings \n Among All Workers") +
	ylab('Share (as %)') +
	theme_bw() +
	scale_x_continuous(breaks = census.years) +
	theme(plot.title = element_text(size = rel(2)))

# Save plots in working directory
ggsave('WageGapEarningsLong.png', width = 5.5, height = 4, units = 'in', scale = 1.5)
ggsave('WageGapEarningsLong.wmf', width = 5.5, height = 4, units = 'in', scale = 1.5)

## Let's now do a latitudinal examination of participation rate in the labor force. Restrict the sample to the 2014 data,
## then calculate the sexes' participation rate across states.

# First, load a CSV file containing state FIP codes
state.fip.code <- read.csv('StateFipCodes.csv')
# Only states are wanted, not regions or unidentified; this selects the desired subset (including DC)
state.fip.code <- state.fip.code[1:51,]

raw.wage.gap.lat = 1:51

raw.gap.data.2012 <- as.data.frame(raw.gap.data[ffwhich(raw.gap.data, raw.gap.data$YEAR == 2012 &
																													raw.gap.data$SEX != 9 &
																													raw.gap.data$EMPSTAT == 1 &
																													raw.gap.data$AGE >= 16),])

# For each state, calculate the wage gap and assign to a vector
for (i in 1:51) {
	# Make verbose
	cat('Calculating medians for i = ', i, '\n')
	# Calculate the weighted median wage for women and men
	women.med.wage <- weightedMedian(raw.gap.data.2012$INCEARN[which(raw.gap.data.2012$SEX == 2 & raw.gap.data.2012$STATEFIP == state.fip.code$STATEFIP[i])], w = raw.gap.data.2012$PERWT[which(raw.gap.data.2012$SEX == 2 & raw.gap.data.2012$STATEFIP == state.fip.code$STATEFIP[i])])
	men.med.wage <- weightedMedian(raw.gap.data.2012$INCEARN[which(raw.gap.data.2012$SEX == 1 & raw.gap.data.2012$STATEFIP == state.fip.code$STATEFIP[i])], w = raw.gap.data.2012$PERWT[which(raw.gap.data.2012$SEX == 1 & raw.gap.data.2012$STATEFIP == state.fip.code$STATEFIP[i])])
	
	# Calculate the wage gap and assign to the appropriate location in raw.wage.gap.lat
	raw.wage.gap.lat[which(raw.wage.gap.lat == i)] <- women.med.wage / men.med.wage
}

#beep()
rm(raw.gap.data.2012)

# Create a data frame containing the data that can be used by ggplot2
raw.wage.gap.lat.data <- data.frame(Gap = raw.wage.gap.lat * 100, State = as.character(state.fip.code$State[1:51]))
raw.wage.gap.lat.data$hl <- 'no'
raw.wage.gap.lat.data$hl[raw.wage.gap.lat.data$State == 'Utah'] <- 'yes'

ggplot(raw.wage.gap.lat.data, aes(y = Gap, x = reorder(State, Gap), fill = hl)) +
	geom_bar(stat = 'identity') +
	scale_fill_manual(values = c('grey20', '#00BFC4'), guide = F) +
	geom_hline(yintercept = raw.wage.gap.nat.long[length(raw.wage.gap.nat.long)] * 100, color = '#F8766D', linetype = 'dashed', size = 1) +
	xlab('State') +
	ylab('Share (as %)') +
	scale_y_continuous(breaks = c(seq(0, 100, by = 25), round(raw.wage.gap.nat.long[length(raw.wage.gap.nat.long)] * 100, digits = 1))) +
	geom_text(aes(label = round(Gap, digits = 1)), vjust = 1.5, size = 2.5, color = 'white') +
	theme_bw() +
	ggtitle("Women's Share of Men's Earnings \n Among All Workers") +
	theme(plot.title = element_text(size = rel(2)),
				axis.text.x = element_text(angle = 45, hjust = 1, vjust = 1))

# Save plots in working directory
ggsave('WageGapEarningsLat.png', width = 9, height = 6, units = 'in', scale = 1.5)
ggsave('WageGapEarningsLat.wmf', width = 9, height = 6, units = 'in', scale = 1.5)
