##############################################
## VfUCRateAverageEduc.r                    ##
##############################################
## Curtis Miller                            ##
## 12/5/14                                  ##
##############################################

## This script calculates the average years of education of women at different levels of
## education. It uses VfUCGenderGap_RawWageGapData1ACS_2013.csv, so be sure that file is
## in the working directory. It is not a small file, so the packages ff and ffbase are
## used to extract data.StateFipsCodes.csv is also needed.

## Aspects calculate include:
##    * Average years of education for women

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages(devtools)
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)

# Load a new CSV file that contains the replicate weights variables
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS_2013.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

paste(rep('REPWTP', times = 80), 1:80, sep = '') -> replicate.vars
selected.variables <- c('STATEFIP', 'SEX', 'PERWT', 'EDUC', 'AGE')

educ.rate.data <- data1[c(selected.variables, replicate.vars)]

# Select full-time year-round workers who are employed, older than 16, who's sex is no NIU (this is the population I will analyze)
educ.rate.data <- educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$AGE >= 25 & educ.rate.data$AGE < 65),]

# Clean up educ.rate.data so unneeded variables are removed
selected.variables <- c('STATEFIP', 'PERWT', 'EDUC', 'SEX')
educ.rate.data <- educ.rate.data[c(selected.variables, replicate.vars)]

calculate.educ.rate <- function(x, data, weight = 'PERWT', verbose = FALSE) {
  total <- sum(as.vector(data[[weight]]))
  
  for (i in x) {
    # Make verbose
    if (verbose == T) {
      cat('Calculating rate for i = ', i, '\n')
    }
    
    # Select the subset of the data corresponding to the selected education
    educ.data <- data[ffwhich(data, data$EDUC == i), weight]
    
    educ.data <- as.vector(educ.data[[weight]])
    # Calculate the rate
    selected <- sum(educ.data)
    
    x[which(x == i)] <- selected / total
  }
  rm(educ.data)
  
  mean.educ <- sum(x * c(0, 4, 8, 9, 10, 11, 12, 13, 14, 16, 17))
  return(mean.educ)
}

calculate.rate.se <- function(x, estimate, verbose = FALSE, data) {
	sum.square = 0
	
	## The formula for calculating the standard errors is
	## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
	## I calculate first the sum of squares.
	for (i in replicate.vars) {
		if (verbose == T) {
			cat('Calculating replicate for i = ', i, '\n')
		}
		sum.square <- sum.square + (calculate.educ.rate(x, data = data, weight = i) - estimate) ^ 2
	}
	
	# Now to calculate the final SE
	se = sqrt(sum.square / 20)
	
	return(se)
}

mean.Utah.men <- calculate.educ.rate(c(0:8, 10:11), data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$STATEFIP == 49 & educ.rate.data$SEX == 1),], verbose = T)
mean.nat.men <- calculate.educ.rate(c(0:8, 10:11), data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$SEX == 1),], verbose = T)

mean.Utah.women <- calculate.educ.rate(c(0:8, 10:11), data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$STATEFIP == 49 & educ.rate.data$SEX == 2),], verbose = T)
mean.nat.women <- calculate.educ.rate(c(0:8, 10:11), data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$SEX == 2),], verbose = T)

mean.Utah.men.se <- calculate.rate.se(c(0:8, 10:11), estimate = mean.Utah.men, data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$STATEFIP == 49 & educ.rate.data$SEX == 1),], verbose = T)
mean.nat.men.se <- calculate.rate.se(c(0:8, 10:11), estimate = mean.nat.men, data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$SEX == 1),], verbose = T)

mean.Utah.women.se <- calculate.rate.se(c(0:8, 10:11), estimate = mean.Utah.women, data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$STATEFIP == 49 & educ.rate.data$SEX == 2),], verbose = T)
mean.nat.women.se <- calculate.rate.se(c(0:8, 10:11), estimate = mean.nat.women, data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$SEX == 2),], verbose = T)

educ.years.data.frame <- data.frame(Years = c(mean.Utah.men, mean.Utah.women, mean.nat.men, mean.nat.women),
				    Level = c('Utah', 'Utah', 'National', 'National'),
				    Sex = c('Male', 'Female', 'Male', 'Female'),
				    SE = c(mean.Utah.men.se, mean.Utah.women.se, mean.nat.men.se, mean.nat.women.se))

write.csv(educ.years.data.frame, 'AverageEducYears.csv')

ggplot(educ.years.data.frame, aes(x = Level, y = Years, fill = Sex)) +
	geom_bar(stat = 'identity', position = position_dodge()) +
	geom_errorbar(aes(x = Level, y = Years, ymin = Years - SE, ymax = Years + SE), position=position_dodge(0.9), width = .5) +
	ggtitle("Average Years of Education Among \n Individuals 25-64, 2013") +
	ylab('Average Years of Education') +
	coord_cartesian(ylim=c(11, 14)) +
	geom_text(aes(x = Level, y = Years, ymax = Years,label = round(Years, digits = 2)), vjust = 1.5, size = 4.5, color = 'white', position=position_dodge(0.9)) +
	theme_bw() +
	theme(plot.title = element_text(size = rel(2)),
	      axis.text.x = element_text(angle = 35, hjust = 1, vjust = 1))

# Save plots in working directory
ggsave('YearsEduc.png', width = 5, height = 4, units = 'in', scale = 1.5)
ggsave('YearsEduc.wmf', width = 5, height = 4, units = 'in', scale = 1.5)
