##############################################
## VfUCRateEducMen.r                        ##
##############################################
## Curtis Miller                            ##
## 11/17/14                                 ##
##############################################

## This script calculates the percentage of men at different levels of education. It uses
## VfUCGenderGap_RawWageGapData1ACS_2013.csv, so be sure that file is in the working directory.
## It is not a small file, so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Percentage of men at level of education

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages(devtools)
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)

# Load a new CSV file that contains the replicate weights variables
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS_2013.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

# Create a vector containing the names of the 80 REPWTPXX vars
paste(rep('REPWTP', times = 80), 1:80, sep = '') -> replicate.vars
selected.variables <- c('STATEFIP', 'SEX', 'PERWT', 'EDUCD', 'AGE')

educ.rate.data <- data1[c(selected.variables, replicate.vars)]

# Select full-time year-round workers who are employed, older than 16, who's sex is no NIU (this is the population I will analyze)
educ.rate.data <- educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$SEX == 1 & educ.rate.data$AGE >= 25 & educ.rate.data$AGE < 65),]

# Clean up educ.rate.data so unneeded variables are removed
selected.variables <- c('STATEFIP', 'PERWT', 'EDUCD')
educ.rate.data <- educ.rate.data[c(selected.variables, replicate.vars)]

# This variable will contain the education rate; first, it contains the left end point of the class
educ.rate.Utah <- c(061, 063, 064, 065, 071, 081, 101, 114, 115, 116)
educ.rate.nat <- educ.rate.Utah

educ.rate.Utah.se <- educ.rate.Utah
educ.rate.nat.se <- educ.rate.nat

calculate.educ.rate <- function(x, data, weight = 'PERWT', verbose = FALSE) {
  total <- sum(as.vector(data[[weight]]))
  
  for (i in x) {
    # Make verbose
    if (verbose == T) {
      cat('Calculating rate for i = ', i, '\n')
    }
    
    # Select the subset of the data corresponding to the selected education
    if (i == 61) {
      educ.data <- data[ffwhich(data, data$EDUCD <= i), weight]
    }
    else {
      educ.data <- data[ffwhich(data, data$EDUCD == i), weight]
    }
    
    educ.data <- as.vector(educ.data[[weight]])
    
    # Calculate the rate
    selected <- sum(educ.data)
    
    x[which(x == i)] <- selected / total
  }
  rm(educ.data)
  
  return(x)
}

## This function takes a row from, say, race.wage.gap.lat.se and a
## subset of race.gap.data.2013. It calculates standard errors using the REPWPTXX
## variables and assigns them to the passed row of the table. x is a vector,
## verbose a logical value, and data an ff object. estimate is a vector that
## contains the estimated value for which a standard error will be calculated.

calculate.rate.se <- function(x, estimate, verbose = FALSE, data) {
  sum.square = rep(0, times = length(x))
  
  ## The formula for calculating the standard errors is
  ## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
  ## I calculate first the sum of squares.
  for (i in replicate.vars) {
    if (verbose == T) {
      cat('Calculating replicate for i = ', i, '\n')
    }
    sum.square <- sum.square + (calculate.educ.rate(x, data = data, weight = i) - estimate) ^ 2
  }
  
  # Now to calculate the final SE
  x = sqrt(sum.square / 20)
  
  return(x)
}

educ.rate.Utah <- calculate.educ.rate(educ.rate.Utah, data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$STATEFIP == 49),], verbose = T)
educ.rate.nat <- calculate.educ.rate(educ.rate.nat, data = educ.rate.data, verbose = T)

educ.rate.Utah.se <- calculate.rate.se(educ.rate.Utah.se, data = educ.rate.data[ffwhich(educ.rate.data, educ.rate.data$STATEFIP == 49),], verbose = T, estimate = educ.rate.Utah)
educ.rate.nat.se <- calculate.rate.se(educ.rate.nat.se, data = educ.rate.data, verbose = T, estimate = educ.rate.nat)

educd.codes <- read.csv('EducCodesDetailed.csv')
educd.labels <- as.character(educd.codes$Educational.attainment[which(educd.codes$EDUCD %in% c(061, 063, 064, 065, 071, 081, 101, 114, 115, 116))])
educd.labels[1] = 'Less than high school diploma'

# Create a dataframe containing the data, and save it to a csv file
educ.rate.data.frame <- data.frame(Utah.Rate = educ.rate.Utah * 100, Nat.Rate = educ.rate.nat * 100, Utah.SE = educ.rate.Utah.se * 100, Nat.Rate.SE = educ.rate.nat.se * 100, Educ = educd.labels)

write.csv(educ.rate.data.frame, file = 'VfUCRateEducMen.csv')

cbPalette <- c("#000000", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7", "#999999", '#D50F64')

ggplot(educ.rate.data.frame) +
  geom_segment(aes(x = 0, xend = 15, y = Utah.Rate, yend = Nat.Rate, color = Educ), size = .75) +
  geom_point(aes(x = 0, y = Utah.Rate, color = Educ), size = 3) +
  geom_point(aes(x = 15, y = Nat.Rate, color = Educ), size = 3) +
  scale_colour_manual(values=cbPalette) +
  ggtitle("Level of Education in 2013 \n Among Men 25-64") +
  xlab('') +
  ylab('') +
  xlim(-5, 20) +
  ylim(0, 1.2*25) +
  geom_text(label = 'Utah', x = 0, y = 1.1*25, size = 5) +
  geom_text(label = 'Nation', x = 15, y = 1.1*25, size = 5) +
  geom_text(aes(label = paste(round(Nat.Rate, digits = 1), '%', sep = ''), x = 15, y = Nat.Rate), hjust = -0.5, size = 2.6) +
  geom_text(aes(label = paste(round(Utah.Rate, digits = 1), '%', sep = ''), x = 0, y = Utah.Rate), hjust = 1.5, size = 2.6) +
  theme(panel.background = element_blank(),
        panel.grid = element_blank(),
        axis.ticks = element_blank(),
        axis.text = element_blank(),
        panel.border = element_blank(),
        axis.title.y = element_text(vjust = 3),
        plot.title = element_text(size = rel(1.5)))

ggsave('EducRateMen.png', width = 5, height = 5, units = 'in', scale = 1.5)
ggsave('EducRateMen.wmf', width = 5, height = 5, units = 'in', scale = 1.5)