##################################################
## VfUCRawGenderGapClosingTimes.r		##
##################################################
## Curtis Miller        	                ##
## 10/16/14                               	##
##################################################

## This script calculates Utah's gender gap over time and creates a regression from it,
## using CPS data.

## This script calculates:
## *  change in gender gap across states
## *  when those gaps will close

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages(devtools)
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)
#library(beepr)
library(matrixStats)	# This contains a function for calculating weighted medians, weigthedMedian()

data <- read.csv.ffdf(file = 'VUCGenderGapClosingTimeCPSData.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

gap.data <- data[ffwhich(data, data$SEX != 9 &
			 	data$AGE >= 16 &
			 	data$EMPSTAT >= 01 &
			 	data$EMPSTAT < 20 &
			 	data$UHRSWORK >= 35 &
			 	data$WKSWORK2 == 6),]

# Create an INCEARN column
gap.data$INCEARN <- as.ff(rep(0, times = length(gap.data$INCWAGE)))
gap.data$INCEARN <- ffvecapply(sapply(i1:i2, function (x) return(gap.data$INCWAGE[x] + gap.data$INCBUS[x] + gap.data$INCFARM[x])), X = gap.data$INCEARN, RETURN = T, VERBOSE = T, BATCHSIZE = 50000)

census.years <- 1977:2014

state.fip.code <- read.csv('StateFipCodes.csv')
# Only states are wanted, not regions or unidentified; this selects the desired subset (including DC)
state.fip.code <- state.fip.code[1:51,]

## This function takes a vector x of state FIP codes, an option to make output
## verbose, and a weight. It returns a vector of wage gaps over time.

calculate.raw.gap <- function(x, verbose = FALSE, weight = 'WTSUPP') {
	if (verbose == T) {
		cat('Calculating medians for x = ', x, '\n')
	}
	# x == 0 means national
	if (x > 0) {
		state.gap.data <- gap.data[ffwhich(gap.data, gap.data$STATEFIP == x),]
	} else {
		state.gap.data <- gap.data
	}
	
	year.data.vec <- census.years
	for (i in census.years) {	# Make verbose
		
		state.gap.data.year <- as.data.frame(state.gap.data[ffwhich(state.gap.data, state.gap.data$YEAR == i),])# Calculate the weighted median wage for women and men

		women.med.wage <- weightedMedian(state.gap.data.year$INCEARN[which(state.gap.data.year$SEX == 2)], w = state.gap.data.year[which(state.gap.data.year$SEX == 2), weight])
		men.med.wage <- weightedMedian(state.gap.data.year$INCEARN[which(state.gap.data.year$SEX == 1)], w = state.gap.data.year[which(state.gap.data.year$SEX == 1), weight])
		
		# Calculate the wage gap and assign to the appropriate location in raw.wage.gap.lat
		year.data.vec[which(year.data.vec == i)] <- women.med.wage / men.med.wage
	}
	
	return(year.data.vec)
}

gaps <- sapply(c(0, state.fip.code$STATEFIP[1:51]), function (x) calculate.raw.gap(x, verbose = T))

gap.data.frame <- data.frame(Gap = as.vector(gaps),
		       Year = rep(census.years, times = 52),
		       State = rep(as.factor(c("National", as.character(state.fip.code$State))), each = length(census.years)))

write.csv(gap.data.frame, 'GenderGapsCPS.csv')

nat.lm.results <- lm(Gap ~ Year, data = gap.data.frame[which(gap.data.frame$State == "National"),])
state.lm.results <- lm(Gap ~ Year + State + Year * State, data = gap.data.frame[which(gap.data.frame$State != 'National'),])

# Making a dataframe with the coefficients and slopes by creating vectors containing the variable names in the linear model, and adding them to the appropriate baseline variable
dummies <- paste(rep('State', times = 50), as.character(state.fip.code$State[2:51]), sep = '')
interacts <- paste(rep('Year:State', times = 50), as.character(state.fip.code$State[2:51]), sep = '')

state.models <- data.frame(Intercept = c(nat.lm.results$coefficients['(Intercept)'], state.lm.results$coefficients['(Intercept)'], state.lm.results$coefficients[dummies] + state.lm.results$coefficients['(Intercept)']),
			   Slope = c(nat.lm.results$coefficients['Year'], state.lm.results$coefficients['Year'], state.lm.results$coefficients[interacts] + state.lm.results$coefficients['Year']),
			   State = c('National', as.character(state.fip.code$State)))

state.models$Year <- 1:52
state.models$Year <- sapply(state.models$Year, function (x) floor((1 - state.models[x, 1]) / state.models[x,2]))
state.models <- state.models[order(state.models$Year),]

write.csv(state.models, 'GapClosureYears.csv')

# Functions for plotting the data
#plot(x = census.years, y = gap.data.frame[which(gap.data.frame$State == 'National'), 'Gap'])
#abline(a = state.models[which(state.models$State == 'National'), 1], b = state.models[which(state.models$State == 'National'), 2])
