##########################################
## VfUCRawGenderGapRace2.r		##
##########################################
## Curtis Miller        	        ##
## 10/22/14                             ##
##########################################

## This script calculates Utah's gender gap (both over time and compared to the rest of the
## nation) across racial groups for full-time year-round workers. It uses
## VfUCGenderGap_RawWageGapData1ACS.csv, so be sure that file is in the working directory,
## and anticipate to spend several minutes waiting for data to load. It is not a small file,
## so the packages ff and ffbase are used to extract data.
## StateFipsCodes.csv is also needed.

## The difference between this script and the original is that women's median earnings are compared
## to all male earnings, not just those males of the same race. Also, there's no latitudinal
## component, due to the complications of comparing when sample sizes are small.

## This script calculates several aspects of participation rates. Aspects calculate
## include:
##    * Raw gender gap across time for different racial groups among full-time workers
##    * Raw gender gap across states for different racial groups among full-time workers
##    * (Others coming soon)

# Run these lines to install necessary packages
#install.packages('ff')
#install.packages('ffbase')
#install.packages('ggplot2')
#install.packages(devtools)
#library(devtools)
#install_github('pingr', 'rasmusab')
#install.packages('matrixStats')

library(ff)
library(ffbase)
library(ggplot2)
#library(beepr)
library(matrixStats)	# This contains a function for calculating weighted medians, weigthedMedian()

# Read in the data from both files for manipulation
data1 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

selected.variables <- c('YEAR', 'STATEFIP', 'SEX', 'EMPSTAT', 'AGE', 'UHRSWORK', 'WKSWORK2', 'PERWT', 'INCEARN', 'RACE', 'HISPAN')
race.gap.data <- data1[selected.variables]
# Clean up race.gap.data to have only those entries with desired values
race.gap.data <- race.gap.data[ffwhich(race.gap.data, race.gap.data$SEX != 9 &
				       	race.gap.data$EMPSTAT == 1 &
				       	race.gap.data$AGE >= 16 &
				       	race.gap.data$UHRSWORK >= 35 &
				       	race.gap.data$WKSWORK2 == 6),]

# race.wage.gap.nat.long will contain the raw wage gap over selected census years for different racial/ethnic/language groups; race.wage.gap.Utah.long is the same for Utah
#census.years <- c(1850, 1860, 1870, 1880, 1900, 1910, 1920, 1930, 1940, 1950, 1960, 1970, 1980, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013)
census.years <- 2000:2013

race.wage.gap.Utah.long <- as.table(matrix(c(census.years, census.years, census.years, census.years), nrow = 4, byrow = T))
colnames(race.wage.gap.Utah.long) <- census.years
rownames(race.wage.gap.Utah.long) <- c('All', 'White non-hispanic', 'Hispanic', 'Non-white non-hispanic')
race.wage.gap.nat.long <- race.wage.gap.Utah.long

## This function takes a row from, say, race.wage.gap.nat.long and a subset of
## race.gap.data. It calculates womens share of men's earnings (women's median
## earnings divided by men's earnings) over years and assigns the results to the
## passed row of the table.

calculate.gap.long <- function(x, verbose = FALSE, data) {
	for (i in census.years) {
		# Make verbose
		if (verbose == TRUE) {
			cat('Calculating medians for i = ', i, '\n')
		}
		# Select the subset of the data corresponding to the selected year
		year.data <- data[ffwhich(data, data$YEAR == i),]
		year.data <- as.data.frame(year.data); gc()
		
		# Calculate the weighted median wage for women
		women.med.wage <- weightedMedian(year.data$INCEARN[which(year.data$SEX == 2)], w = year.data$PERWT[which(year.data$SEX == 2)])
		
		# Calculate the weighted median for all men in a given year
		year.data2 <- as.data.frame(race.gap.data[ffwhich(race.gap.data, race.gap.data$YEAR == i & race.gap.data$SEX == 1 & race.gap.data$STATEFIP == 49),]) ## WATCH IT!!!
		men.med.wage <- weightedMedian(year.data2$INCEARN, w = year.data2$PERWT)
		
		# Calculate the wage gap and assign to the appropriate location in x
		x[which(x == i)] <- women.med.wage / men.med.wage
		
		rm(year.data); rm(year.data2)
	}
	return(x)
}

## Now, fill in the tables.

# First, Utah data
race.wage.gap.Utah.long['All',] <- calculate.gap.long(race.wage.gap.Utah.long['All',], verbose = T, data = race.gap.data[ffwhich(race.gap.data, race.gap.data$STATEFIP == 49),])
race.wage.gap.Utah.long['White non-hispanic',] <- calculate.gap.long(race.wage.gap.Utah.long['White non-hispanic',], data = race.gap.data[ffwhich(race.gap.data, race.gap.data$STATEFIP == 49 &
																		  	race.gap.data$RACE == 1 &
																		  	race.gap.data$HISPAN == 0),])
race.wage.gap.Utah.long['Hispanic',] <- calculate.gap.long(race.wage.gap.Utah.long['Hispanic',], verbose = T, data = race.gap.data[ffwhich(race.gap.data, race.gap.data$STATEFIP == 49 &
																	   	race.gap.data$HISPAN != 0),])
race.wage.gap.Utah.long['Non-white non-hispanic',] <- calculate.gap.long(race.wage.gap.Utah.long['Non-white non-hispanic',], verbose = T, data = race.gap.data[ffwhich(race.gap.data, race.gap.data$STATEFIP == 49 &
																				       	race.gap.data$RACE != 1 &
																				       	race.gap.data$HISPAN == 0),])

# Next, national data
race.wage.gap.nat.long['All',] <- calculate.gap.long(race.wage.gap.nat.long['All',], data = race.gap.data, verbose = T)
race.wage.gap.nat.long['White non-hispanic',] <- calculate.gap.long(race.wage.gap.nat.long['White non-hispanic',], verbose = T, data = race.gap.data[ffwhich(race.gap.data, race.gap.data$RACE == 1 &
																			     	race.gap.data$HISPAN == 0),])
race.wage.gap.nat.long['Hispanic',] <- calculate.gap.long(race.wage.gap.nat.long['Hispanic',], verbose = T, data = race.gap.data[ffwhich(race.gap.data, race.gap.data$HISPAN != 0),])
race.wage.gap.nat.long['Non-white non-hispanic',] <- calculate.gap.long(race.wage.gap.nat.long['Non-white non-hispanic',], verbose = T, data = race.gap.data[ffwhich(race.gap.data, race.gap.data$RACE != 1 &
																				     	race.gap.data$HISPAN == 0),])

## I want to make a chart that plots these values over time, including both
## national and Utah data. I create a dataframe that lays the tables flat,
## differentiates them based on category and whether they are state or national.

race.wage.gap.long.data <- data.frame(Gap = c(as.vector(race.wage.gap.Utah.long), as.vector(race.wage.gap.nat.long)) * 100,
				      Year = rep(census.years, each = 4, times = 2),
				      Race = rep(c('All', 'White non-hispanic', 'Hispanic', 'Non-white non-hispanic'), times = 28),
				      Level = rep(c('Utah', 'National'), each = length(as.vector(race.wage.gap.Utah.long))))

# Save the calculated results
write.csv(race.wage.gap.long.data, file = 'VfUCRawGenderGapRaceLong2.csv')

# Draw the plot
ggplot(race.wage.gap.long.data) +
	geom_line(aes(x = Year, y = Gap, color = Race, linetype = Level), size = 1.1) +
	scale_linetype_manual(values = c('dashed', 'solid')) +
	#	ylim(50, 100) +
	ggtitle("Women's Share of All Men's Earnings \n Among Full-Time Year-Round Workers, By Race") +
	ylab("Share (as %)") +
	theme_bw() +
	scale_x_continuous(breaks = census.years) +
	theme(plot.title = element_text(size = rel(1.2)))

# Save plots in working directory
ggsave('RaceEarningsGapLong2.png', width = 5.5, height = 4, units = 'in', scale = 1.5)
ggsave('RaceEarningsGapLong2.wmf', width = 5.5, height = 4, units = 'in', scale = 1.5)

########################################################################################
########################################################################################
########################################################################################

## Let's now repeat these calculations, only latitudinally. For latitudinal statistics,
## I want to calculate standard errors along with the desired statistic. The file
## VfUCGenderGap_RawWageGapData1ACS_2013.csv is identical to
## VfUCGenderGap_RawWageGapData1ACS.csv, but includes the REPWTXX variables for
## computing standard errors directly from the data. This script utilizes this to
## calculate standard errors for the 2013 data (due to the size of the resulting files,
## this is not calculated for other years; there are major technical limitations).

data2 <- read.csv.ffdf(file = 'VfUCGenderGap_RawWageGapData1ACS_2013.csv', first.rows = 10000, next.rows = 50000, VERBOSE = T, colClasses = NA)

## Create a vector containing the names of the 80 REPWTPXX vars
paste(rep('REPWTP', times = 80), 1:80, sep = '') -> replicate.vars
race.gap.data.2013 <- data2[c(selected.variables, replicate.vars)]
# Clean up race.gap.data.2013 to have only those entries with desired values
race.gap.data.2013 <- race.gap.data.2013[ffwhich(race.gap.data.2013, race.gap.data.2013$SEX != 9 &
																								 	race.gap.data.2013$EMPSTAT == 1 &
																								 	race.gap.data.2013$AGE >= 16 &
																								 	race.gap.data.2013$UHRSWORK >= 35 &
																								 	race.gap.data.2013$WKSWORK2 == 6),]

# First, load a CSV file containing state FIP codes
state.fip.code <- read.csv('StateFipCodes.csv')
# Only states are wanted, not regions or unidentified; this selects the desired subset (including DC)
state.fip.code <- state.fip.code[1:51,]

race.wage.gap.lat <- as.table(matrix(c(state.fip.code$STATEFIP, state.fip.code$STATEFIP, state.fip.code$STATEFIP, state.fip.code$STATEFIP), nrow = 4, byrow = T))
colnames(race.wage.gap.lat) <- state.fip.code$State
rownames(race.wage.gap.lat) <- c('All', 'White non-hispanic', 'Hispanic', 'Non-white non-hispanic')

# Create a similar table, only for storing standard errors
race.wage.gap.lat.se <- race.wage.gap.lat

## This function takes a row from, say, race.wage.gap.lat and a subset of
## race.gap.data. It calculates womens share of men's earnings (women's median
## earnings divided by men's earnings) over states and assigns the results to the
## passed row of the table. x is a vector, verbose a logical value, data an ff
## object, and weight a string

calculate.gap.lat <- function(x, verbose = FALSE, data, weight = 'PERWT') {
	for (i in state.fip.code$STATEFIP) {
		# Make verbose
		if (verbose == TRUE) {
			cat('Calculating medians for i = ', i, '\n')
		}
		# Select the subset of the data corresponding to the selected year
		state.data <- as.data.frame(data[ffwhich(data, data$STATEFIP == i), c('INCEARN', 'SEX', weight)])
		
		# Calculate the weighted median wage for women
		women.med.wage <- weightedMedian(state.data$INCEARN[which(state.data$SEX == 2)], w = state.data[which(state.data$SEX == 2), weight])

		# Calculate the weighted median wage for all men
		
		state.data2 <- as.data.frame(race.gap.data.2013[ffwhich(race.gap.data.2013, race.gap.data.2013$SEX == 1 & race.gap.data.2013$STATEFIP == i), c('INCEARN', weight)])
		men.med.wage <- weightedMedian(state.data2$INCEARN, w = state.data2[,weight])
		
		# Calculate the wage gap and assign to the appropriate location in x
		x[which(x == i)] <- women.med.wage / men.med.wage
		
		rm(state.data); rm(state.data2)
	}
	return(x)
}

## This function takes a row from, say, race.wage.gap.lat.se and a
## subset of race.gap.data.2013. It calculates standard errors using the REPWPTXX
## variables and assigns them to the passed row of the table. x is a vector,
## verbose a logical value, and data an ff object. estimate is a vector that
## contains the estimated value for which a standard error will be calculated.

calculate.gap.se <- function(x, estimate, verbose = FALSE, data) {
	sum.square = rep(0, times = length(x))
	
	## The formula for calculating the standard errors is
	## SE(X) = sqrt(4/80 * sum(from r = 1 to 80, (X_r - X)^2))
	## I calculate first the sum of squares.
	for (i in replicate.vars) {
		if (verbose == T) {
			cat('Calculating replicate for i = ', i, '\n')
		}
		sum.square <- sum.square + (calculate.gap.lat(x, data = data, weight = i) - estimate) ^ 2
	}
	
	# Now to calculate the final SE
	x = sqrt(sum.square / 20)
	
	return(x)
}

race.wage.gap.lat['All',] <- calculate.gap.lat(race.wage.gap.lat['All',], verbose = T, data = race.gap.data.2013)
race.wage.gap.lat['White non-hispanic',] <- calculate.gap.lat(race.wage.gap.lat['White non-hispanic',], data = race.gap.data.2013[ffwhich(race.gap.data.2013, race.gap.data.2013$RACE == 1 &
																																																																						race.gap.data.2013$HISPAN == 0),])
race.wage.gap.lat['Hispanic',] <- calculate.gap.lat(race.wage.gap.lat['Hispanic',], verbose = T, data = race.gap.data.2013[ffwhich(race.gap.data.2013, race.gap.data.2013$HISPAN != 0),])
race.wage.gap.lat['Non-white non-hispanic',] <- calculate.gap.lat(race.wage.gap.lat['Non-white non-hispanic',], verbose = T, data = race.gap.data.2013[ffwhich(race.gap.data.2013, race.gap.data.2013$RACE != 1 & 
																																																																															 	race.gap.data.2013$HISPAN == 0),])
# Calculate standard errors

race.wage.gap.lat.se['All',] <- calculate.gap.se(race.wage.gap.lat.se['All',],
																								 estimate = race.wage.gap.lat['All',],
																								 verbose = T, 
																								 data = race.gap.data.2013)
race.wage.gap.lat.se['White non-hispanic',] <- calculate.gap.se(race.wage.gap.lat.se['White non-hispanic',],
																																estimate = race.wage.gap.lat['White non-hispanic',], verbose = T,
																																data = race.gap.data.2013[ffwhich(race.gap.data.2013, race.gap.data.2013$RACE == 1 & 
																																																		race.gap.data.2013$HISPAN == 0),])
race.wage.gap.lat.se['Hispanic',] <- calculate.gap.se(race.wage.gap.lat.se['Hispanic',],
																											estimate = race.wage.gap.lat['Hispanic',],
																											verbose = T,
																											data = race.gap.data.2013[ffwhich(race.gap.data.2013, race.gap.data.2013$HISPAN != 0),])
race.wage.gap.lat.se['Non-white non-hispanic',] <- calculate.gap.se(race.wage.gap.lat.se['Non-white non-hispanic',],
																																		estimate = race.wage.gap.lat['Non-white non-hispanic',],
																																		verbose = T,
																																		data = race.gap.data.2013[ffwhich(race.gap.data.2013, race.gap.data.2013$RACE != 1 & 
																																																				race.gap.data.2013$HISPAN == 0),])

# Prepare to display in a graphic

race.wage.gap.lat.data <- data.frame(Gap = as.vector(race.wage.gap.lat) * 100,
																		 SE = as.vector(race.wage.gap.lat.se) * 100,
																		 State = rep(state.fip.code$State, each = 4),
																		 Race = rep(c('All', 'White non-hispanic', 'Hispanic', 'Non-white non-hispanic'), times = 51))

write.csv(race.wage.gap.lat.data, file = 'VfUCRawGenderGapRaceLat2.csv')

# Draw a Cleveland dot plot
ggplot(race.wage.gap.lat.data) +
	geom_errorbarh(aes(x = Gap, y = reorder(State, Gap), xmin = Gap - SE, xmax = Gap + SE, color = Race), linetype = 'twodash') +
	geom_point(aes(x = Gap, y = reorder(State, Gap), color = Race), size = 4) +
	geom_hline(yintercept = 4, color = 'orange', size = 3, alpha = .2) +
	theme_bw() +
	ggtitle("Women's Share of All Men's Earnings Among \n Full-Time Year-Round Workers, By Race, 2013") +
	xlab('Share (as %)') +
	ylab('State') +
	theme(panel.grid.major.x = element_blank(),
				panel.grid.minor.x = element_blank(),
				panel.grid.major.y = element_line(color = 'grey60', linetype = 'dotted'),
				plot.title = element_text(size = rel(1.2)))

# Save plots in working directory
ggsave('RaceEarningsGapLat2.png', width = 10, height = 8, units = 'in', scale = 1.5)
ggsave('RaceEarningsGapLat2.wmf', width = 10, height = 8, units = 'in', scale = 1.5)
